// ESP8266 JSON from Mega2560
//https://forum.arduino.cc/t/arduino-mega-to-esp8266-communication-via-tx-rx/857016/67

// download SafeString V4.1.0+ library from the Arduino Library manager or from
// https://www.forward.com.au/pfod/ArduinoProgramming/SafeString/index.html

#include <ESP8266WebServer.h>
#include <ESP8266WiFi.h>
#include <WiFiClient.h>
#include <ArduinoJson.h>
#include "SerialComs.h"
#include "SoftwareSerial.h"
// include the web page html
#include "PageIndex.h"

const int RX_pin = 13; // for ESP8266 use 13  D7 on wemos-d1-esp8266
const int TX_pin = 12; // for ESP8266 use 12  D6  on wemos-d1-esp8266
SoftwareSerial toESP(RX_pin, TX_pin);

// sendLineLength, receiveLineLength
SerialComs coms(10, 250); // send 10 (nothing sent), receive 250 chars
// receiveLineLength must match other sides sendLineLength
// sendLineLength must match other sides receiveLineLength

StaticJsonDocument<1000> doc;

const char* ssid = "xxxxx";
const char* password = "xxxxx";

ESP8266WebServer server(80);

// these are the snapshot values
float Vin;
float Iin;
float Pin;
float Vout;
float Iout;
float Pout;

void handle_Root() {
  server.send(200, "text/html", MAIN_page); //Send web page
}

void handle_NotFound() {
  server.send(404, "text/plain", "Page not found");
}

void formatValue(SafeString& result, double value) {
  result.clear();
  result.print(value, 3, 7, true);
}

// send snapshot of the current data
void handle_Snapshot() {
  cSF(sfResult, 45);
  cSF(sfData, 100); // data to send

  formatValue(sfResult, Vin);  sfData.print(sfResult);  sfData.print(',');
  formatValue(sfResult, Vin);  sfData.print(sfResult);  sfData.print(',');
  formatValue(sfResult, Iin);  sfData.print(sfResult);  sfData.print(',');
  formatValue(sfResult, Pin);  sfData.print(sfResult);  sfData.print(',');
  formatValue(sfResult, Vout);  sfData.print(sfResult);  sfData.print(',');
  formatValue(sfResult, Iout);  sfData.print(sfResult);  sfData.print(',');
  formatValue(sfResult, Pout);  sfData.print(sfResult);
  //Serial.println(sfData);
  server.send(200, "text/plane", sfData.c_str());
}

void setup() {
  Serial.begin(115200);
  for (int i = 10; i > 0; i--) {
    Serial.print(' ');
    Serial.print(i);
    delay(500);
  }
  Serial.println();
  Serial.println("Jelka Bisa MPPT Project V, I and P measurements");

  //SafeString::setOutput(Serial); // enable error messages and debugging

  toESP.begin(9600, SWSERIAL_8N1, -1, -1, false, 256); // use previous rxPin, txPin and set 256 RX buffer
  coms.setAsController(); // always set one side (and only one side) as the controller
  // The slowest loop code should be set at the controller, usually the web side

  if (!coms.connect(toESP)) {
    while (1) {
      Serial.println(F("Out of memory"));
    }
  }
  Serial.println(F("ESP8266 Setup finished."));

  WiFi.begin(ssid, password);
  Serial.print("Connecting network .");
  while (WiFi.status() != WL_CONNECTED) {
    Serial.print(".");
    delay(500);
  }
  Serial.println(" done");
  Serial.print("Successfully connected to : ");
  Serial.println(ssid);
  Serial.print("IP address: ");
  Serial.println(WiFi.localIP());

  server.on("/", handle_Root); //--> This is to display web page.
  server.on("/getSnapshotData", handle_Snapshot); //--> handle the call procedure getSnapshotData
  server.begin(); //--> Start server
  Serial.println("HTTP server started");

  // if you want a startup msg
  // coms.textToSend = F("StartedUp"); // tell the other side we have booted up
}

void loop() {
  coms.sendAndReceive(); // must do this every loop

  if (!coms.textReceived.isEmpty()) { // got some data

    DeserializationError error = deserializeJson(doc, coms.textReceived.c_str());
    if (error) {
      Serial.print(F(("deserializeJson() failed with code - ")));
      Serial.println(error.c_str());
    } else {
      //Fetching values
      Vin = doc["Vin"];
      Iin = doc["Iin"];
      Pin = doc["Pin"];
      Vout = doc["Vout"];
      Iout = doc["Iout"];
      Pout = doc["Pout"];
    }
  }
  server.handleClient();
}
